% This script simulate a signal (convolution of a few number of spikes and 
% a Gaussian kernel and performs the localization of spikes.
% It is inspired from the numerical tours by Gabriel Peyre.
% http://www.numerical-tours.com/matlab/sparsity_9_sparsespikes_cbp/
% 
% Hayato Ikoma, 2015
% Stanford University

addpath('function')

%% Set parameters
N = 32; % For sampling
NN = N*8; % For true signal
Delta = 1/N;

% Define functional handles that we use later
phi = @(t,sigma)exp(-t.^2/(2*sigma^2));
phi1 = @(t,sigma)-t/(sigma^2).*exp(-t.^2/(2*sigma^2));
convol  = @(x,h)real(ifft(fft(x).*fft(h)));
convolS = @(x,h)real(ifft(fft(x).*conj(fft(h))));

% Set the grid
t = [0:N/2, -N/2+1:-1]' / N;
t1 = (0:N-1)'/N;

tt = [0:NN/2, -NN/2+1:-1]' / NN;
tt1 = (0:NN-1)'/NN;
 

%% Generate observation signal
Phi  = @(x,sigma)convol(x,phi(t,sigma));
sigma0 = [1 1]/N; % std. for Gaussian kernels
a0_val = [1 .8]; % intensity of spikes
d0_val = [.3 .9]; % shift from the grid
K = length(sigma0); % number of spikes

% I = round( N/(2*K):N/K:N );
I = [15 18];

a0 = zeros(N,1);
d0 = zeros(N,1);

for k=1:K
    a0(I(k)) = a0_val(k);
    d0(I(k)) = d0_val(k);    
end
x0 = (0:N-1)'/N + d0 * Delta/2;
b0 = d0.*a0;

y = zeros(N,1); yy = zeros(NN,1);
for k = 1:K
    sp_pos = (I(k)-1)/N + d0_val(k)*Delta/2;
    T = t - sp_pos; TT = tt - sp_pos;
    T = mod(T,1); T(T>.5) = T(T>.5)-1;
    TT = mod(TT,1); TT(TT>.5) = TT(TT>.5)-1;    
    y = y + a0_val(k)*phi(T,sigma0(k));
    yy = yy + a0_val(k)*phi(TT,sigma0(k));
end

% visualize the simulated signal
figure;
plot(t1,y,'o'); hold on;
plot(tt1,yy,'-');


%% Estiamte the locations with CSC-CBP
sigmaList = [1/N]; % std. of Gaussian kernels (arbitrary number is fine.)
alpha = 2;
num_kernel = length(sigmaList);

[estLoc,hist] = upsampleCBPCSC1D(y,alpha,sigmaList);

% Visualize
lw = 2; msB = 20;
mystem = @(x,y,cc)stem(x, y, '.--','Color',cc, 'MarkerSize', msB, 'LineWidth', lw);
cmap = get(groot,'DefaultAxesColorOrder');

c = estLoc.c;
loc_x = estLoc.loc_x;

figure; hold on;
plot(tt1,yy,'-','Color',cmap(1,:))
plot(t1,y,'o','MarkerSize',10,'Color',cmap(1,:));
xlim([0 1])
mystem(x0(I),a0(I),cmap(2,:));
mystem(loc_x,c,cmap(5,:));
xlabel('Location')
ylabel('Intensity')
